@@ -109,12 +109,6 @@ dh_app_search_assistant (DhApp *self,
         g_action_group_activate_action (G_ACTION_GROUP (self), "search-assistant", g_variant_new_string (keyword));
 }
 
-void
-dh_app_focus_search (DhApp *self)
-{
-        g_action_group_activate_action (G_ACTION_GROUP (self), "focus-search", NULL);
-}
-
 void
 dh_app_raise (DhApp *self)
 {
@@ -242,19 +236,6 @@ search_assistant_cb (GSimpleAction *action,
         dh_assistant_search (DH_ASSISTANT (assistant), str);
 }
 
-static void
-focus_search_cb (GSimpleAction *action,
-                 GVariant      *parameter,
-                 gpointer       user_data)
-{
-        DhApp *self = DH_APP (user_data);
-        GtkWindow *window;
-
-        window = dh_app_peek_first_window (self);
-        dh_window_focus_search (DH_WINDOW (window));
-        gtk_window_present (window);
-}
-
 static void
 raise_cb (GSimpleAction *action,
           GVariant      *parameter,
@@ -277,7 +258,6 @@ static GActionEntry app_entries[] = {
         /* additional commandline-specific actions */
         { "search",           search_cb,           "s",  NULL, NULL },
         { "search-assistant", search_assistant_cb, "s",  NULL, NULL },
-        { "focus-search",     focus_search_cb,     NULL, NULL, NULL },
         { "raise",            raise_cb,            NULL, NULL, NULL },
 };
 
@@ -291,29 +271,37 @@ setup_actions (DhApp *self)
 
 /******************************************************************************/
 
+static void
+setup_accelerators (DhApp *self)
+{
+        gtk_application_add_accelerator (GTK_APPLICATION (self), "<Primary>0",     "win.zoom-default", NULL);
+        gtk_application_add_accelerator (GTK_APPLICATION (self), "<Primary>minus", "win.zoom-out",     NULL);
+        gtk_application_add_accelerator (GTK_APPLICATION (self), "<Primary>plus",  "win.zoom-in",      NULL);
+        gtk_application_add_accelerator (GTK_APPLICATION (self), "<Primary>f",     "win.find",         NULL);
+        gtk_application_add_accelerator (GTK_APPLICATION (self), "<Primary>c",     "win.copy",         NULL);
+        gtk_application_add_accelerator (GTK_APPLICATION (self), "<Primary>p",     "win.print",        NULL);
+        gtk_application_add_accelerator (GTK_APPLICATION (self), "<Primary>t",     "win.new-tab",      NULL);
+}
+
+/******************************************************************************/
+
 static void
 setup_menu (DhApp *self)
 {
         GtkBuilder *builder;
         GMenuModel *model;
-        gchar *path;
         GError *error = NULL;
 
         builder = gtk_builder_new ();
 
-        path = dh_util_build_data_filename ("devhelp", "ui", "devhelp.builder", NULL);
-        if (!gtk_builder_add_from_file (builder, path, &error)) {
-                g_error ("Cannot create builder from '%s': %s",
-                         path, error ? error->message : "unknown error");
+        if (!gtk_builder_add_from_resource (builder, "/org/gnome/devhelp/devhelp.ui", &error)) {
+                g_error ("%s",error ? error->message : "unknown error");
+                g_clear_error (&error);
         }
 
         model = G_MENU_MODEL (gtk_builder_get_object (builder, "app-menu"));
         gtk_application_set_app_menu (GTK_APPLICATION (self), model);
 
-        model = G_MENU_MODEL (gtk_builder_get_object (builder, "window-menu"));
-        gtk_application_set_menubar (GTK_APPLICATION (self), model);
-
-        g_free (path);
         g_object_unref (builder);
 }
 
@@ -331,6 +319,9 @@ startup (GApplication *application)
         /* Setup menu */
         setup_menu (self);
 
+        /* Setup accelerators */
+        setup_accelerators (self);
+
         /* Load the book manager */
         g_assert (self->priv->book_manager == NULL);
         self->priv->book_manager = dh_book_manager_new ();
