@@ -20,13 +20,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/*
- * Fullscreen mode code adapted from gedit
- *  Copyright (C) 1998, 1999 Alex Roberts, Evan Lawrence
- *  Copyright (C) 2000, 2001 Chema Celorio, Paolo Maggi
- *  Copyright (C) 2002-2005 Paolo Maggi
- */
-
 #include "config.h"
 #include <string.h>
 #include <math.h>
@@ -39,39 +32,29 @@
 #include <webkit/webkit.h>
 #endif
 
-#ifdef GDK_WINDOWING_QUARTZ
-#include <gtkosxapplication.h>
-#endif
+#include <libgd/gd.h>
 
-#include "dh-book-tree.h"
 #include "dh-book-manager.h"
 #include "dh-book.h"
-#include "dh-search.h"
+#include "dh-sidebar.h"
 #include "dh-window.h"
 #include "dh-util.h"
-#include "dh-marshal.h"
 #include "dh-enum-types.h"
 #include "dh-settings.h"
 #include "eggfindbar.h"
 
-#define FULLSCREEN_ANIMATION_SPEED 4
 #define TAB_WIDTH_N_CHARS 15
 
 struct _DhWindowPriv {
         GtkWidget      *main_box;
         GtkWidget      *hpaned;
-        GtkWidget      *control_notebook;
-        GtkWidget      *book_tree;
-        GtkWidget      *search;
+        GtkWidget      *sidebar;
         GtkWidget      *notebook;
+        GtkWidget      *toolbar;
 
         GtkWidget      *vbox;
         GtkWidget      *findbar;
 
-        GtkWidget      *fullscreen_controls;
-        guint           fullscreen_animation_timeout_id;
-        gboolean        fullscreen_animation_enter;
-
         GtkBuilder     *builder;
         GtkActionGroup *action_group;
 
@@ -120,9 +103,6 @@ static const guint n_zoom_levels = G_N_ELEMENTS (zoom_levels);
 static void           dh_window_class_init           (DhWindowClass   *klass);
 static void           dh_window_init                 (DhWindow        *window);
 static void           window_populate                (DhWindow        *window);
-static void           window_tree_link_selected_cb   (GObject         *ignored,
-                                                      DhLink          *link,
-                                                      DhWindow        *window);
 static void           window_search_link_selected_cb (GObject         *ignored,
                                                       DhLink          *link,
                                                       DhWindow        *window);
@@ -165,9 +145,6 @@ static void           window_close_tab               (DhWindow *window,
                                                       gint      page_num);
 static gboolean       do_search                      (DhWindow *window);
 
-static void           window_fullscreen_controls_build (DhWindow *window);
-static void           window_fullscreen_controls_show (DhWindow *window);
-
 G_DEFINE_TYPE (DhWindow, dh_window, GTK_TYPE_APPLICATION_WINDOW);
 
 #define GET_PRIVATE(instance) G_TYPE_INSTANCE_GET_PRIVATE \
@@ -250,7 +227,7 @@ copy_cb (GSimpleAction *action,
         if (GTK_IS_EDITABLE (widget)) {
                 gtk_editable_copy_clipboard (GTK_EDITABLE (widget));
         } else if (GTK_IS_TREE_VIEW (widget) &&
-                   gtk_widget_is_ancestor (widget, priv->search) &&
+                   gtk_widget_is_ancestor (widget, priv->sidebar) &&
                    priv->selected_search_link) {
                 GtkClipboard *clipboard;
                 clipboard = gtk_widget_get_clipboard (widget, GDK_SELECTION_CLIPBOARD);
@@ -339,38 +316,23 @@ window_get_current_zoom_level_index (DhWindow *window)
 static void
 window_update_zoom_actions_state (DhWindow *window)
 {
-        DhWindowPriv *priv;
         GAction *action;
         int zoom_level_idx;
         gboolean enabled;
 
-        priv = window->priv;
-
         zoom_level_idx = window_get_current_zoom_level_index (window);
 
         enabled = zoom_levels[zoom_level_idx].level < ZOOM_MAXIMAL;
         action = g_action_map_lookup_action (G_ACTION_MAP (window), "zoom-in");
         g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
-        if (priv->fullscreen_controls) {
-                action = g_action_map_lookup_action (G_ACTION_MAP (priv->fullscreen_controls), "zoom-in");
-                g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
-        }
 
         enabled = zoom_levels[zoom_level_idx].level > ZOOM_MINIMAL;
         action = g_action_map_lookup_action (G_ACTION_MAP (window), "zoom-out");
         g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
-        if (priv->fullscreen_controls) {
-                action = g_action_map_lookup_action (G_ACTION_MAP (priv->fullscreen_controls), "zoom-out");
-                g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
-        }
 
         enabled = zoom_levels[zoom_level_idx].level != ZOOM_DEFAULT;
         action = g_action_map_lookup_action (G_ACTION_MAP (window), "zoom-default");
         g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
-        if (priv->fullscreen_controls) {
-                action = g_action_map_lookup_action (G_ACTION_MAP (priv->fullscreen_controls), "zoom-default");
-                g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
-        }
 }
 
 static void
@@ -423,45 +385,6 @@ zoom_default_cb (GSimpleAction *action,
         window_update_zoom_actions_state (window);
 }
 
-static gboolean
-window_is_fullscreen (DhWindow *window)
-{
-        GdkWindowState  state;
-
-        g_return_val_if_fail (DH_IS_WINDOW (window), FALSE);
-
-        state = gdk_window_get_state (gtk_widget_get_window (GTK_WIDGET (window)));
-
-        return state & GDK_WINDOW_STATE_FULLSCREEN;
-}
-
-static void
-window_fullscreen (DhWindow *window)
-{
-        if (window_is_fullscreen (window))
-                return;
-
-        gtk_window_fullscreen (GTK_WINDOW (window));
-        gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (window->priv->builder, "toolbar")));
-        gtk_application_window_set_show_menubar (GTK_APPLICATION_WINDOW (window), FALSE);
-
-        window_fullscreen_controls_build (window);
-        window_fullscreen_controls_show (window);
-}
-
-static void
-window_unfullscreen (DhWindow *window)
-{
-        if (! window_is_fullscreen (window))
-                return;
-
-        gtk_window_unfullscreen (GTK_WINDOW (window));
-        gtk_widget_show (GTK_WIDGET (gtk_builder_get_object (window->priv->builder, "toolbar")));
-        gtk_application_window_set_show_menubar (GTK_APPLICATION_WINDOW (window), TRUE);
-
-        gtk_widget_hide (window->priv->fullscreen_controls);
-}
-
 static void
 go_back_cb (GSimpleAction *action,
             GVariant      *parameter,
@@ -501,34 +424,6 @@ go_forward_cb (GSimpleAction *action,
         webkit_web_view_go_forward (web_view);
 }
 
-static void
-go_contents_tab_cb (GSimpleAction *action,
-                    GVariant      *parameter,
-                    gpointer       user_data)
-{
-        DhWindow     *window = user_data;
-        DhWindowPriv *priv;
-
-        priv = window->priv;
-
-        gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->control_notebook), 0);
-        gtk_widget_grab_focus (priv->book_tree);
-}
-
-static void
-go_search_tab_cb (GSimpleAction *action,
-                  GVariant      *parameter,
-                  gpointer       user_data)
-{
-        DhWindow     *window = user_data;
-        DhWindowPriv *priv;
-
-        priv = window->priv;
-
-        gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->control_notebook), 1);
-        gtk_widget_grab_focus (priv->search);
-}
-
 static void
 window_open_link_cb (DhWindow *window,
                      const char *location,
@@ -542,45 +437,6 @@ window_open_link_cb (DhWindow *window,
         }
 }
 
-static void
-fullscreen_cb (GSimpleAction *action,
-               GVariant      *parameter,
-               gpointer       data)
-{
-        GVariant *state = g_action_get_state (G_ACTION (action));
-        gboolean value = g_variant_get_boolean (state);
-
-        g_action_change_state (G_ACTION (action), g_variant_new_boolean (!value));
-        g_variant_unref (state);
-}
-
-static void
-fullscreen_change_state_cb (GSimpleAction *action,
-                            GVariant      *state,
-                            gpointer       data)
-{
-        DhWindow *self = data;
-        gboolean is_active = g_variant_get_boolean (state);
-
-        if (is_active)
-                window_fullscreen (self);
-        else
-                window_unfullscreen (self);
-        g_simple_action_set_state (action, state);
-}
-
-static void
-leave_fullscreen_cb (GSimpleAction *action,
-                     GVariant      *parameter,
-                     gpointer       data)
-{
-        DhWindow *self = data;
-        GAction *fs_action;
-
-        fs_action = g_action_map_lookup_action (G_ACTION_MAP (self), "fullscreen");
-        g_action_change_state (G_ACTION (fs_action), g_variant_new_boolean (FALSE));
-}
-
 static GActionEntry win_entries[] = {
         /* file */
         { "new-tab",          new_tab_cb,          NULL, NULL, NULL },
@@ -595,224 +451,11 @@ static GActionEntry win_entries[] = {
         { "zoom-in",          zoom_in_cb,          NULL, NULL, NULL },
         { "zoom-out",         zoom_out_cb,         NULL, NULL, NULL },
         { "zoom-default",     zoom_default_cb,     NULL, NULL, NULL },
-        { "fullscreen",       fullscreen_cb,       NULL, "false", fullscreen_change_state_cb },
-        { "leave-fullscreen", leave_fullscreen_cb, NULL, NULL, NULL },
         /* go */
         { "go-back",          go_back_cb,          NULL, "false", NULL },
         { "go-forward",       go_forward_cb,       NULL, "false", NULL },
-        { "go-contents-tab",  go_contents_tab_cb,  NULL, NULL, NULL },
-        { "go-search-tab",    go_search_tab_cb,    NULL, NULL, NULL },
 };
 
-static gboolean
-run_fullscreen_animation (gpointer data)
-{
-	DhWindow *window = DH_WINDOW (data);
-	GdkScreen *screen;
-	GdkRectangle fs_rect;
-	gint x, y;
-
-	screen = gtk_window_get_screen (GTK_WINDOW (window));
-	gdk_screen_get_monitor_geometry (screen,
-					 gdk_screen_get_monitor_at_window (screen,
-									   gtk_widget_get_window (GTK_WIDGET (window))),
-					 &fs_rect);
-
-	gtk_window_get_position (GTK_WINDOW (window->priv->fullscreen_controls),
-				 &x, &y);
-
-	if (window->priv->fullscreen_animation_enter)
-	{
-		if (y == fs_rect.y)
-		{
-			window->priv->fullscreen_animation_timeout_id = 0;
-			return FALSE;
-		}
-		else
-		{
-			gtk_window_move (GTK_WINDOW (window->priv->fullscreen_controls),
-					 x, y + 1);
-			return TRUE;
-		}
-	}
-	else
-	{
-		gint w, h;
-
-		gtk_window_get_size (GTK_WINDOW (window->priv->fullscreen_controls),
-				     &w, &h);
-
-		if (y == fs_rect.y - h + 1)
-		{
-			window->priv->fullscreen_animation_timeout_id = 0;
-			return FALSE;
-		}
-		else
-		{
-			gtk_window_move (GTK_WINDOW (window->priv->fullscreen_controls),
-					 x, y - 1);
-			return TRUE;
-		}
-	}
-}
-
-static void
-show_hide_fullscreen_toolbar (DhWindow *window,
-                              gboolean     show,
-                              gint         height)
-{
-        GtkSettings *settings;
-        gboolean enable_animations;
-
-        settings = gtk_widget_get_settings (GTK_WIDGET (window));
-        g_object_get (G_OBJECT (settings),
-                      "gtk-enable-animations",
-                      &enable_animations,
-                      NULL);
-
-        if (enable_animations)
-        {
-                window->priv->fullscreen_animation_enter = show;
-
-                if (window->priv->fullscreen_animation_timeout_id == 0)
-                {
-                        window->priv->fullscreen_animation_timeout_id =
-                                g_timeout_add (FULLSCREEN_ANIMATION_SPEED,
-                                               (GSourceFunc) run_fullscreen_animation,
-                                               window);
-                }
-        }
-        else
-        {
-                GdkRectangle fs_rect;
-                GdkScreen *screen;
-
-                screen = gtk_window_get_screen (GTK_WINDOW (window));
-                gdk_screen_get_monitor_geometry (screen,
-                                                 gdk_screen_get_monitor_at_window (screen,
-                                                                                   gtk_widget_get_window (GTK_WIDGET (window))),
-                                                 &fs_rect);
-
-                if (show)
-                        gtk_window_move (GTK_WINDOW (window->priv->fullscreen_controls),
-                                         fs_rect.x, fs_rect.y);
-                else
-                        gtk_window_move (GTK_WINDOW (window->priv->fullscreen_controls),
-                                         fs_rect.x, fs_rect.y - height + 1);
-        }
-
-}
-
-static gboolean
-on_fullscreen_controls_enter_notify_event (GtkWidget        *widget,
-                                           GdkEventCrossing *event,
-                                           DhWindow      *window)
-{
-        show_hide_fullscreen_toolbar (window, TRUE, 0);
-
-        return FALSE;
-}
-
-static gboolean
-on_fullscreen_controls_leave_notify_event (GtkWidget        *widget,
-                                           GdkEventCrossing *event,
-                                           DhWindow      *window)
-{
-        GdkDisplay *display;
-        GdkScreen *screen;
-        gint w, h;
-        gint x, y;
-
-        display = gdk_display_get_default ();
-        screen = gtk_window_get_screen (GTK_WINDOW (window));
-
-        gtk_window_get_size (GTK_WINDOW (window->priv->fullscreen_controls), &w, &h);
-        gdk_display_get_pointer (display, &screen, &x, &y, NULL);
-
-        /* gtk seems to emit leave notify when clicking on tool items,
-         * work around it by checking the coordinates
-         */
-        if (y >= h)
-        {
-                show_hide_fullscreen_toolbar (window, FALSE, h);
-        }
-
-        return FALSE;
-}
-
-static void
-window_fullscreen_controls_build (DhWindow *window)
-{
-        GtkWidget *toolbar;
-        DhWindowPriv  *priv;
-
-        priv = window->priv;
-        if (priv->fullscreen_controls != NULL)
-                return;
-
-        /* Note: the Fullscreen Controls window needs to be a
-         * GtkApplicationWindow, in order to be able to process GActions.
-         * Moreover, the window needs to get destroyed with the parent, or
-         * we'll end up leaving a reference to the GtkApplication around and
-         * it will not get closed properly. */
-        priv->fullscreen_controls = g_object_new (GTK_TYPE_APPLICATION_WINDOW,
-                                                  "type", GTK_WINDOW_POPUP,
-                                                  "application", gtk_window_get_application (GTK_WINDOW (window)),
-                                                  "show-menubar", FALSE,
-                                                  NULL);
-        gtk_window_set_transient_for (GTK_WINDOW (priv->fullscreen_controls),
-                                      GTK_WINDOW (window));
-        gtk_window_set_destroy_with_parent (GTK_WINDOW (priv->fullscreen_controls),
-                                            TRUE);
-
-        toolbar = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-                                                      "fullscreen-toolbar"));
-        gtk_container_add (GTK_CONTAINER (priv->fullscreen_controls),
-                           toolbar);
-
-        /* Set the toolbar style */
-        gtk_toolbar_set_style (GTK_TOOLBAR (toolbar),
-                               GTK_TOOLBAR_BOTH_HORIZ);
-
-        g_signal_connect (priv->fullscreen_controls, "enter-notify-event",
-                          G_CALLBACK (on_fullscreen_controls_enter_notify_event),
-                          window);
-        g_signal_connect (priv->fullscreen_controls, "leave-notify-event",
-                          G_CALLBACK (on_fullscreen_controls_leave_notify_event),
-                          window);
-
-        g_action_map_add_action_entries (G_ACTION_MAP (priv->fullscreen_controls),
-                                         win_entries, G_N_ELEMENTS (win_entries),
-                                         window);
-
-        window_check_history (window, NULL);
-}
-
-static void
-window_fullscreen_controls_show (DhWindow *window)
-{
-        GdkScreen *screen;
-        GdkRectangle fs_rect;
-        gint w, h;
-
-        screen = gtk_window_get_screen (GTK_WINDOW (window));
-        gdk_screen_get_monitor_geometry (screen,
-                                         gdk_screen_get_monitor_at_window (
-                                                 screen,
-                                                 gtk_widget_get_window (GTK_WIDGET (window))),
-                                         &fs_rect);
-
-        gtk_window_get_size (GTK_WINDOW (window->priv->fullscreen_controls), &w, &h);
-
-        gtk_window_resize (GTK_WINDOW (window->priv->fullscreen_controls),
-                           fs_rect.width, h);
-
-        gtk_window_move (GTK_WINDOW (window->priv->fullscreen_controls),
-                         fs_rect.x, fs_rect.y - h + 1);
-
-        gtk_widget_show_all (window->priv->fullscreen_controls);
-}
-
 static void
 settings_fonts_changed_cb (DhSettings *settings,
                            const gchar *font_name_fixed,
@@ -854,12 +497,13 @@ dh_window_init (DhWindow *window)
         GtkAccelGroup *accel_group;
         GClosure      *closure;
         gint           i;
-        gchar         *path;
         GError        *error = NULL;
 
         priv = GET_PRIVATE (window);
         window->priv = priv;
 
+        gtk_window_set_hide_titlebar_when_maximized (GTK_WINDOW (window), TRUE);
+
         priv->selected_search_link = NULL;
 
         /* handle settings */
@@ -871,12 +515,10 @@ dh_window_init (DhWindow *window)
 
         /* Setup builder */
         priv->builder = gtk_builder_new ();
-        path = dh_util_build_data_filename ("devhelp", "ui", "devhelp.builder", NULL);
-        if (!gtk_builder_add_from_file (priv->builder, path, &error)) {
-                g_error ("Cannot create builder from '%s': %s",
-                         path, error ? error->message : "unknown error");
+        if (!gtk_builder_add_from_resource (priv->builder, "/org/gnome/devhelp/devhelp.ui", &error)) {
+                g_error ("Cannot add resource to builder: %s", error ? error->message : "unknown error");
+                g_clear_error (&error);
         }
-        g_free (path);
 
         priv->main_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
         gtk_widget_show (priv->main_box);
@@ -911,6 +553,7 @@ dispose (GObject *object)
 {
 	DhWindow *self = DH_WINDOW (object);
         g_clear_object (&self->priv->settings);
+        g_clear_object (&self->priv->builder);
 
 	/* Chain up to the parent class */
 	G_OBJECT_CLASS (dh_window_parent_class)->dispose (object);
@@ -929,7 +572,7 @@ dh_window_class_init (DhWindowClass *klass)
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (DhWindowClass, open_link),
                               NULL, NULL,
-                              _dh_marshal_VOID__STRING_FLAGS,
+                              g_cclosure_marshal_generic,
                               G_TYPE_NONE,
                               2,
                               G_TYPE_STRING,
@@ -946,45 +589,6 @@ dh_window_class_init (DhWindowClass *klass)
                              "style \"devhelp-tab-close-button-style\"");
 }
 
-/* The ugliest hack. When switching tabs, the selection and cursor is changed
- * for the tree view so the web_view content is changed. Block the signal during
- * switch.
- */
-static void
-window_control_switch_page_cb (GtkWidget       *notebook,
-                               gpointer         page,
-                               guint            page_num,
-                               DhWindow        *window)
-{
-        DhWindowPriv *priv;
-
-        priv = window->priv;
-
-        g_signal_handlers_block_by_func (priv->book_tree,
-                                         window_tree_link_selected_cb,
-                                         window);
-}
-
-static void
-window_control_after_switch_page_cb (GtkWidget       *notebook,
-                                     gpointer         page,
-                                     guint            page_num,
-                                     DhWindow        *window)
-{
-        DhWindowPriv *priv;
-
-        priv = window->priv;
-
-        g_signal_handlers_unblock_by_func (priv->book_tree,
-                                           window_tree_link_selected_cb,
-                                           window);
-        /* save the current selected tab */
-        const gchar *label = gtk_notebook_get_tab_label_text (GTK_NOTEBOOK (notebook), page);
-        g_settings_set_string (
-                dh_settings_peek_search_notebook_settings (priv->settings),
-                "selected-tab", label);
-}
-
 static void
 window_web_view_switch_page_cb (GtkNotebook     *notebook,
                                 gpointer         page,
@@ -1003,13 +607,12 @@ window_web_view_switch_page_cb (GtkNotebook     *notebook,
 
                 new_web_view = g_object_get_data (G_OBJECT (new_page), "web_view");
 
-                /* Sync the book tree. */
+                /* Sync the book tree */
                 location = webkit_web_view_get_uri (new_web_view);
 
-                if (location) {
-                        dh_book_tree_select_uri (DH_BOOK_TREE (priv->book_tree),
-                                                 location);
-                }
+                if (location)
+                        dh_sidebar_select_uri (DH_SIDEBAR (priv->sidebar), location);
+
                 window_check_history (window, new_web_view);
 
                 window_update_title (window, new_web_view, NULL);
@@ -1034,115 +637,59 @@ static void
 window_populate (DhWindow *window)
 {
         DhWindowPriv  *priv;
-        GtkWidget     *book_tree_sw;
         DhBookManager *book_manager;
-        GtkWidget     *toolbar;
-        gchar         *selected_tab;
-        guint         i;
+        GtkWidget     *back;
+        GtkWidget     *forward;
+        GtkWidget     *box;
+        GtkWidget     *menu_button;
+        GObject       *menu;
 
         priv = window->priv;
-
-        toolbar = GTK_WIDGET (gtk_builder_get_object (priv->builder, "toolbar"));
-
-        /* Add toolbar to main box */
-        gtk_style_context_add_class (gtk_widget_get_style_context (toolbar),
-                                     GTK_STYLE_CLASS_PRIMARY_TOOLBAR);
-        gtk_box_pack_start (GTK_BOX (priv->main_box), toolbar,
-                            FALSE, FALSE, 0);
-
-#ifdef GDK_WINDOWING_QUARTZ
-        {
-                GtkOSXApplication *theApp;
-
-                /* Hide toolbar labels. */
-                gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
-
-                /* Setup menubar. */
-                theApp = g_object_new (GTK_TYPE_OSX_APPLICATION, NULL);
-                gtk_osxapplication_set_menu_bar (theApp, GTK_MENU_SHELL (menubar));
-
-                g_signal_connect (theApp,
-                                  "NSApplicationWillTerminate",
-                                  G_CALLBACK (window_activate_quit),
-                                  window);
-
-                gtk_osxapplication_ready (theApp);
-        }
-#endif
-
-        priv->hpaned = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
-
-        gtk_box_pack_start (GTK_BOX (priv->main_box), priv->hpaned, TRUE, TRUE, 0);
-
-        /* Search and contents notebook. */
-        priv->control_notebook = gtk_notebook_new ();
-
-        gtk_paned_add1 (GTK_PANED (priv->hpaned), priv->control_notebook);
-
-        book_tree_sw = gtk_scrolled_window_new (NULL, NULL);
-
-        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (book_tree_sw),
-                                        GTK_POLICY_NEVER,
-                                        GTK_POLICY_AUTOMATIC);
-        gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (book_tree_sw),
-                                             GTK_SHADOW_IN);
-        gtk_container_set_border_width (GTK_CONTAINER (book_tree_sw), 2);
-
         book_manager = dh_app_peek_book_manager (DH_APP (gtk_window_get_application (GTK_WINDOW (window))));
 
-        priv->book_tree = dh_book_tree_new (book_manager);
-        gtk_container_add (GTK_CONTAINER (book_tree_sw),
-                           priv->book_tree);
-        gtk_notebook_append_page (GTK_NOTEBOOK (priv->control_notebook),
-                                  book_tree_sw,
-                                  gtk_label_new (_("Contents")));
-        g_signal_connect (priv->book_tree,
-                          "link-selected",
-                          G_CALLBACK (window_tree_link_selected_cb),
-                          window);
+        priv->toolbar = gd_main_toolbar_new ();
+        back = gd_main_toolbar_add_button (GD_MAIN_TOOLBAR (priv->toolbar),
+                                           "go-previous-symbolic",
+                                           _("Back"),
+                                           TRUE);
+        gtk_actionable_set_action_name ( GTK_ACTIONABLE (back), "win.go-back");
+        forward = gd_main_toolbar_add_button (GD_MAIN_TOOLBAR (priv->toolbar),
+                                              "go-next-symbolic",
+                                              _("Forward"),
+                                              TRUE);
+        gtk_actionable_set_action_name ( GTK_ACTIONABLE (forward), "win.go-forward");
+        box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+        gtk_style_context_add_class (gtk_widget_get_style_context (box), "linked");
+        gtk_widget_reparent (back, box);
+        gtk_widget_reparent (forward, box);
 
-        priv->search = dh_search_new (book_manager);
-        gtk_notebook_append_page (GTK_NOTEBOOK (priv->control_notebook),
-                                  priv->search,
-                                  gtk_label_new (_("Search")));
-        g_signal_connect (priv->search,
+        gd_main_toolbar_add_widget (GD_MAIN_TOOLBAR (priv->toolbar), box, TRUE);
+        menu_button = gd_main_toolbar_add_menu (GD_MAIN_TOOLBAR (priv->toolbar),
+                                                "emblem-system-symbolic",
+                                                "",
+                                                FALSE);
+        menu = gtk_builder_get_object (priv->builder, "window-menu");
+        gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (menu_button), G_MENU_MODEL (menu));
+
+        /* Add toolbar to main box */
+        gtk_box_pack_start (GTK_BOX (priv->main_box), priv->toolbar,
+                            FALSE, FALSE, 0);
+
+        priv->hpaned = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
+        gtk_box_pack_start (GTK_BOX (priv->main_box), priv->hpaned, TRUE, TRUE, 0);
+
+        /* Sidebar */
+        priv->sidebar = dh_sidebar_new (book_manager);
+        gtk_paned_add1 (GTK_PANED (priv->hpaned), priv->sidebar);
+        g_signal_connect (priv->sidebar,
                           "link-selected",
                           G_CALLBACK (window_search_link_selected_cb),
                           window);
 
+        /* Document view */
         priv->vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
         gtk_paned_add2 (GTK_PANED (priv->hpaned), priv->vbox);
 
-        g_signal_connect (priv->control_notebook,
-                          "switch-page",
-                          G_CALLBACK (window_control_switch_page_cb),
-                          window);
-
-        g_signal_connect_after (priv->control_notebook,
-                                "switch-page",
-                                G_CALLBACK (window_control_after_switch_page_cb),
-                                window);
-
-        /* restore selected control notebook page */
-        selected_tab = g_settings_get_string (
-                dh_settings_peek_search_notebook_settings (priv->settings),
-                "selected-tab");
-        for (i = 0; i < gtk_notebook_get_n_pages (GTK_NOTEBOOK (priv->control_notebook)); i++) {
-                GtkWidget   *page;
-                const gchar *page_name;
-
-                page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (priv->control_notebook), i);
-                page_name = gtk_notebook_get_tab_label_text (GTK_NOTEBOOK (priv->control_notebook), page);
-
-                if (page_name && strcmp (page_name, selected_tab) == 0) {
-                        gtk_widget_set_visible (GTK_WIDGET (page), TRUE);
-                        gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->control_notebook), i);
-                        gtk_widget_grab_focus (page);
-                        break;
-                }
-        }
-        g_free (selected_tab);
-
         /* HTML tabs notebook. */
         priv->notebook = gtk_notebook_new ();
         gtk_container_set_border_width (GTK_CONTAINER (priv->notebook), 0);
@@ -1159,7 +706,7 @@ window_populate (DhWindow *window)
                                 G_CALLBACK (window_web_view_switch_page_after_cb),
                                 window);
 
-        /* Create findbar. */
+        /* Create findbar */
         priv->findbar = egg_find_bar_new ();
         gtk_widget_set_no_show_all (priv->findbar, TRUE);
         gtk_box_pack_start (GTK_BOX (priv->vbox), priv->findbar, FALSE, FALSE, 0);
@@ -1252,13 +799,11 @@ window_web_view_navigation_policy_decision_requested (WebKitWebView
                                                       DhWindow                  *window)
 #endif
 {
-        DhWindowPriv *priv;
         const char   *uri;
 #ifdef HAVE_WEBKIT2
         WebKitNavigationPolicyDecision *navigation_decision;
 #endif
 
-        priv = window->priv;
 
 #ifdef HAVE_WEBKIT2
         if (type != WEBKIT_POLICY_DECISION_TYPE_NAVIGATION_ACTION)
@@ -1312,14 +857,40 @@ window_web_view_navigation_policy_decision_requested (WebKitWebView
                 return TRUE;
         }
 
+#ifndef HAVE_WEBKIT2
+        /* We already do this in load_changed_cb() for webkit2 */
         if (web_view == window_get_active_web_view (window)) {
-                dh_book_tree_select_uri (DH_BOOK_TREE (priv->book_tree), uri);
+                DhWindowPriv *priv;
+
+                priv = window->priv;
+                dh_sidebar_select_uri (DH_SIDEBAR (priv->sidebar), uri);
                 window_check_history (window, web_view);
         }
+#endif
 
         return FALSE;
 }
 
+#ifdef HAVE_WEBKIT2
+static void
+window_web_view_load_changed_cb (WebKitWebView   *web_view,
+                                 WebKitLoadEvent  load_event,
+                                 DhWindow        *window)
+{
+        const gchar *uri;
+        DhWindowPriv *priv;
+
+        priv = window->priv;
+
+        if (load_event != WEBKIT_LOAD_COMMITTED)
+                return;
+
+        uri = webkit_web_view_get_uri (web_view);
+        dh_sidebar_select_uri (DH_SIDEBAR (priv->sidebar), uri);
+        window_check_history (window, web_view);
+}
+#endif
+
 #ifdef HAVE_WEBKIT2
 static gboolean
 window_web_view_load_failed_cb (WebKitWebView   *web_view,
@@ -1363,23 +934,6 @@ window_web_view_load_error_cb (WebKitWebView  *web_view,
         return TRUE;
 }
 
-static void
-window_tree_link_selected_cb (GObject  *ignored,
-                              DhLink   *link,
-                              DhWindow *window)
-{
-        WebKitWebView *view;
-        gchar         *uri;
-
-        view = window_get_active_web_view (window);
-
-        uri = dh_link_get_uri (link);
-        webkit_web_view_load_uri (view, uri);
-        g_free (uri);
-
-        window_check_history (window, view);
-}
-
 static void
 window_search_link_selected_cb (GObject  *ignored,
                                 DhLink   *link,
@@ -1408,25 +962,14 @@ window_check_history (DhWindow      *window,
 {
         GAction       *action;
         gboolean       enabled;
-        DhWindowPriv  *priv;
-
-        priv = window->priv;
 
         enabled = web_view ? webkit_web_view_can_go_forward (web_view) : FALSE;
         action = g_action_map_lookup_action (G_ACTION_MAP (window), "go-forward");
         g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
-        if (priv->fullscreen_controls) {
-                action = g_action_map_lookup_action (G_ACTION_MAP (priv->fullscreen_controls), "go-forward");
-                g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
-        }
 
         enabled = web_view ? webkit_web_view_can_go_back (web_view) : FALSE;
         action = g_action_map_lookup_action (G_ACTION_MAP (window), "go-back");
         g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
-        if (priv->fullscreen_controls) {
-                action = g_action_map_lookup_action (G_ACTION_MAP (priv->fullscreen_controls), "go-back");
-                g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
-        }
 }
 
 static void
@@ -1741,6 +1284,9 @@ window_open_new_tab (DhWindow    *window,
                           window);
 #endif
 #ifdef HAVE_WEBKIT2
+        g_signal_connect (view, "load-changed",
+                          G_CALLBACK (window_web_view_load_changed_cb),
+                          window);
         g_signal_connect (view, "load-failed",
                           G_CALLBACK (window_web_view_load_failed_cb),
                           window);
@@ -1752,7 +1298,9 @@ window_open_new_tab (DhWindow    *window,
 
         num = gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook),
                                         vbox, NULL);
-
+        gtk_container_child_set (GTK_CONTAINER (priv->notebook), vbox,
+                                 "tab-expand", TRUE,
+                                 NULL);
         gtk_notebook_set_tab_label (GTK_NOTEBOOK (priv->notebook),
                                     vbox, label);
 
@@ -1775,7 +1323,6 @@ window_open_new_tab (DhWindow    *window,
         return num;
 }
 
-#ifndef GDK_WINDOWING_QUARTZ
 static void
 close_button_clicked_cb (GtkButton *button,
                          DhWindow  *window)
@@ -1794,46 +1341,12 @@ close_button_clicked_cb (GtkButton *button,
         }
 }
 
-static void
-tab_label_style_set_cb (GtkWidget *hbox,
-			GtkStyle *previous_style,
-			gpointer user_data)
-{
-	PangoFontMetrics *metrics;
-	PangoContext *context;
-	GtkStyleContext *style;
-	GtkWidget *button;
-	int char_width, h, w;
-
-	context = gtk_widget_get_pango_context (hbox);
-	style = gtk_widget_get_style_context (hbox);
-	metrics = pango_context_get_metrics (context,
-					     gtk_style_context_get_font (style, GTK_STATE_FLAG_NORMAL),
-					     pango_context_get_language (context));
-	char_width = pango_font_metrics_get_approximate_digit_width (metrics);
-	pango_font_metrics_unref (metrics);
-
-	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (hbox),
-					   GTK_ICON_SIZE_MENU, &w, &h);
-
-	gtk_widget_set_size_request
-		(hbox, TAB_WIDTH_N_CHARS * PANGO_PIXELS(char_width) + 2 * w, -1);
-
-	button = g_object_get_data (G_OBJECT (hbox), "close-button");
-	gtk_widget_set_size_request (button, w + 2, h + 2);
-}
-#endif
-
-/* Don't create a close button on quartz, it looks very much out of
- * place.
- */
 static GtkWidget*
 window_new_tab_label (DhWindow        *window,
                       const gchar     *str,
                       const GtkWidget *parent)
 {
         GtkWidget *label;
-#ifndef GDK_WINDOWING_QUARTZ
         GtkWidget *hbox;
         GtkWidget *close_button;
         GtkWidget *image;
@@ -1859,21 +1372,10 @@ window_new_tab_label (DhWindow        *window,
 
         gtk_box_pack_start (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
 
-        /* Set minimal size */
-        g_signal_connect (hbox, "style-set",
-                          G_CALLBACK (tab_label_style_set_cb),
-                          NULL);
-
         g_object_set_data (G_OBJECT (hbox), "label", label);
         g_object_set_data (G_OBJECT (hbox), "close-button", close_button);
 
         return hbox;
-#else
-        label = gtk_label_new (str);
-        g_object_set_data (G_OBJECT (label), "label", label);
-
-        return label;
-#endif
 }
 
 static WebKitWebView *
@@ -1920,7 +1422,6 @@ window_update_title (DhWindow      *window,
                      const gchar   *web_view_title)
 {
         DhWindowPriv *priv;
-        const gchar  *book_title;
 
         priv = window->priv;
 
@@ -1931,27 +1432,8 @@ window_update_title (DhWindow      *window,
                 web_view_title = NULL;
         }
 
-        book_title = dh_book_tree_get_selected_book_title (DH_BOOK_TREE (priv->book_tree));
-
-        /* Don't use both titles if they are the same. */
-        if (book_title && web_view_title && strcmp (book_title, web_view_title) == 0) {
-                web_view_title = NULL;
-        }
-
-        if (!book_title) {
-                /* i18n: Please don't translate "Devhelp" (it's marked as translatable
-                 * for transliteration only) */
-                book_title = _("Devhelp");
-        }
-
-        if (web_view_title) {
-                gchar *full_title;
-                full_title = g_strdup_printf ("%s - %s", book_title, web_view_title);
-                gtk_window_set_title (GTK_WINDOW (window), full_title);
-                g_free (full_title);
-        } else {
-                gtk_window_set_title (GTK_WINDOW (window), book_title);
-        }
+        gd_main_toolbar_set_labels (GD_MAIN_TOOLBAR (priv->toolbar),
+                web_view_title, NULL);
 }
 
 static void
@@ -2032,21 +1514,7 @@ dh_window_search (DhWindow    *window,
 
         priv = window->priv;
 
-        gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->control_notebook), 1);
-        dh_search_set_search_string (DH_SEARCH (priv->search), str, NULL);
-}
-
-void
-dh_window_focus_search (DhWindow *window)
-{
-        DhWindowPriv *priv;
-
-        g_return_if_fail (DH_IS_WINDOW (window));
-
-        priv = window->priv;
-
-        gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->control_notebook), 1);
-        gtk_widget_grab_focus (priv->search);
+        dh_sidebar_set_search_string (DH_SIDEBAR (priv->sidebar), str);
 }
 
 /* Only call this with a URI that is known to be in the docs. */
@@ -2064,5 +1532,5 @@ _dh_window_display_uri (DhWindow    *window,
 
         web_view = window_get_active_web_view (window);
         webkit_web_view_load_uri (web_view, uri);
-        dh_book_tree_select_uri (DH_BOOK_TREE (priv->book_tree), uri);
+        dh_sidebar_select_uri (DH_SIDEBAR (priv->sidebar), uri);
 }
